@@ -13,18 +13,21 @@ use Crashplan::Client::User;
 use Crashplan::Client::Computer;
 use Crashplan::Client::ComputerUsage;
 use Crashplan::Client::MountPoint;
+use Crashplan::Client::ServerStatistics;
+
+use Data::Dumper;
 
 =head1 NAME
 
-Crashplan::Client - The great new Crashplan::Client!
+Crashplan::Client - Client to the Crashplan PROe server 
 
 =head1 VERSION
 
-Version 0.01
+Version 0.002.0
 
 =cut
 
-our $VERSION = '0.001_1';
+our $VERSION = '0.002_0';
 
 =head1 SYNOPSIS
 
@@ -32,11 +35,33 @@ Crashplan::Client allow you to access an Crashplan PROe server (hopefully) in a
 
 This version only provides a low level API matching part of the server REST API.
 
-The main objective in a near future is to provide an additional higher level API
-(read 'syntaxic sugar') while enlarging the coverage of the lowlevel API.
+This version (0.2.0) provides the new highlevel API which will add syntaxic sugar
+and more functional goodies in the near future.
+
+What you can do now :
+
+    use Crashplan::Client;
+
+    my $client = Crashplan::Client->new();
+    
+    my @orgs = $client->orgs;
+
+    my $org = shift @orgs;
+
+    ...
+
+It's planned to offer (NOT IMPLEMENTED YET) something more like :
+
+    use Crashplan::Client;
+
+    my $client = Crashplan::Client->new();
+    
+    my $org = $client->orgs->first;
+
+    ...
 
 
-Now you (still) must do 
+The first lowlevel API is still present 
 
     use Crashplan::Client;
 
@@ -51,33 +76,9 @@ Now you (still) must do
     ...
 
 
-It's planned to offer (NOT IMPLEMENTED YET) something more like :
+=head1 SUBROUTINES/METHODS - Highlevel API
 
-
-    use Crashplan::Client;
-
-    my $client = Crashplan::Client->new();
-    
-    my @orgs = $client->orgs;
-
-    my $org = shift @orgs;
-
-    ...
-
- or even
-
-    use Crashplan::Client;
-
-    my $client = Crashplan::Client->new();
-    
-    my $org = $client->orgs->first;
-
-    ...
-
-
-=head1 SUBROUTINES/METHODS
-
-=head2 new
+=head2 new ()
 
 Constructor for the Crashplan::Client class
 
@@ -102,13 +103,128 @@ sub new {
 
     if ( $self->{server} ) {
         $self->{rest} = REST::Client->new();
+        # Automatically follow redirect
+        $self->{rest}->setFollow(1);
         $self->{rest}->setHost($self->{server});
     }
 
     return bless $self, $class;
 }
 
-=head2 set_rest_header
+=head2 users ()
+
+    Return all the users entity from the server
+
+    Input  : None
+
+    Output : An array of Crashplan::Client::User
+
+=cut
+
+sub users {
+    my $self = shift;
+    
+    $self->GET('/rest/users');
+
+    return $self->parse_response;
+}
+
+=head2 orgs ()
+
+    Return all the orgs entity from the server
+
+    Input  : None
+
+    Output : An array of Crashplan::Client::Org
+
+=cut
+
+sub orgs {
+    my $self = shift;
+    
+    $self->GET('/rest/orgs');
+
+    return $self->parse_response;
+}
+
+=head2 computers ()
+
+    Return all the computers entity from the server
+
+    Input  : None
+
+    Output : An array of Crashplan::Client::Computer
+
+=cut
+
+sub computers {
+    my $self = shift;
+    
+    $self->GET('/rest/computers');
+
+    return $self->parse_response;
+}
+
+=head2 user ($id)
+
+    Return the user entity whose id is passed as parameter 
+
+    Input  : None
+
+    Output : A Crashplan::Client::User object
+
+=cut
+
+sub user {
+    my $self = shift;
+    my $id   = shift;
+    
+    $self->GET("/rest/users/$id");
+
+    return $self->parse_response;
+}
+
+=head2 computer ($id)
+
+    Return the computer entity whose id is passed as parameter 
+
+    Input  : None
+
+    Output : A Crashplan::Client::Computer object
+
+=cut
+
+sub computer {
+    my $self = shift;
+    my $id   = shift;
+    
+    $self->GET("/rest/computers/$id");
+
+    return $self->parse_response;
+}
+
+=head2 org ($id)
+
+    Return the org entity whose id is passed as parameter 
+
+    Input  : None
+
+    Output : A Crashplan::Client::Org object
+
+=cut
+
+sub org {
+    my $self = shift;
+    my $id   = shift;
+    
+    $self->GET("/rest/orgs/$id");
+
+    return $self->parse_response;
+}
+
+=head1 SUBROUTINES/METHODS - Lowlevel API
+
+=head2 set_rest_header ($key, $value)
 
     Set a rest header
 
@@ -126,11 +242,11 @@ sub set_rest_header {
     $self->{rest_header}{$key} = $value;
 }
 
-=head2 unset_rest_header
+=head2 unset_rest_header ($key)
 
     Unset a rest header
 
-    Input  : header the name and the value of the header to be unset
+    Input  : $key the name of the header to be unset
 
     Output : None
 
@@ -144,13 +260,17 @@ sub unset_rest_header {
     delete $self->{rest_header}{$key};
 }
 
-=head2 request
+=head2 request ($method, $url [,$content, $header_ref])
 
     Request against the rest API
 
-    Input  : method
+    Input  :    $method the method to be used (GET, POST, PUT, DELETE)
+                $url the url to be used with the server
+                $content (OPTIONAL) content of the request
+                $header  (OPTIONAL) hash reference of a header
 
     Output : None
+             Will set internal attributes responseCode and responseContent
 
 =cut
 
@@ -161,18 +281,16 @@ sub request {
     my $content = shift || '' ;
     my $header  = shift || $self->default_header;
 
-    croak "Server undefined" unless $self->{server};
-
-    $self->{'rest'}->request( $method, $self->{server}.$url, $content, $header );
+    $self->{'rest'}->request( $method, $url, $content, $header );
 }
 
-=head2 responseContent
+=head2 responseContent ()
 
     Get the response content (for the previous request)
 
     Input  : None
 
-    Output : None
+    Output : A string with the response as a JSON structure
 
 =cut
 
@@ -182,13 +300,13 @@ sub responseContent {
     return $self->{'rest'}->responseContent;
 }
 
-=head2 responseCode
+=head2 responseCode ()
 
     Get the response code (for the previous request)
 
     Input  : None
 
-    Output : None
+    Output : An integer
 
 =cut
 
@@ -198,9 +316,11 @@ sub responseCode {
     return $self->{'rest'}->responseCode;
 }
 
-=head2 default_header
+=head2 default_header ()
 
     Build a default header based on $self object attribute
+    In particular user and password attributes are used to
+    build the Basic Authentication credentials.
 
     Input  : None
 
@@ -220,13 +340,15 @@ sub default_header {
 
 }
 
-=head2 GET
+=head2 GET ($url [,$header])
 
     GET request against the REST server
 
     Input  : $url the url to be requested
 
     Output : None
+             The state of the response is store in the internal 'rest' attribute which is 
+             currently an REST::Client object
 
 =cut
 
@@ -239,13 +361,14 @@ sub GET {
 }
 
 
-=head2 parse_response
+=head2 parse_response ()
 
     Parse a server response to populate Crashplan objects
 
     Input  : None (use $self->responseContent) 
 
-    Output : None
+    Output : Array or single Crashplan::Client::<entity> object based on the 
+             previous request answer
 
 =cut
 
@@ -253,9 +376,13 @@ sub parse_response {
     my $self = shift;
     my $response = shift || $self->responseContent;
 
-    $self->{responses} = decode_json($response);
+    #$self->{responses} = decode_json($response);
+    $self->{responses} = from_json($response);
 
 
+    #
+    # If a list is returned 2 keys at least are available metadata and the requested entity
+    #
     for my $entity (keys %{$self->{responses}}) {
         if ($entity =~ /orgs/) {
             return _populate('Crashplan::Client::Org', $self->{responses}{$entity});
@@ -269,12 +396,26 @@ sub parse_response {
             return _populate('Crashplan::Client::MountPoint', $self->{responses}{$entity});
         }
     }
+    #
+    # If only a single entity try to guess type based on specific attribute
+    #
+
+    if (exists $self->{responses}{parentId}) {
+            return _populate('Crashplan::Client::Org', $self->{responses});
+    } elsif ($self->{responses}{mountPointId}) {
+            return _populate('Crashplan::Client::Computer', $self->{responses});
+    } elsif ($self->{responses}{email}) {
+            return _populate('Crashplan::Client::User', $self->{responses});
+    } elsif ($self->{responses}{cpuUtilization}) {
+            return _populate('Crashplan::Client::ServerStatistics', $self->{responses});
+    };
+
     
 }
 
-=head2 _populate
+=head2 _populate ($entity_name, $hashref)
 
-    Return an array of Crashplan::Client object from a hash.
+    Return an array of Crashplan::Client::$entity_name objects from the $hashref.
 
     Input  : Class name
 
@@ -284,16 +425,20 @@ sub parse_response {
 
 sub _populate {
     my $class = shift;
-    my $aref  = shift;
+    my $ref   = shift;
 
     my @result;
-
-    foreach my $entity (@$aref) {
-        my $object = $class->new($entity);
-        push @result, $object;
+    if (ref($ref) =~ /ARRAY/) {
+        foreach my $entity (@$ref) {
+            my $object = $class->new($entity);
+            push @result, $object;
+        }
+        return @result;
+    } elsif (ref($ref) =~ /HASH/) {
+        my $object = $class->new($ref);
+        return $object;
     }
 
-    return @result;
 }
 
 =head1 TESTING
@@ -358,6 +503,12 @@ by the Free Software Foundation; or the Artistic License.
 See http://dev.perl.org/licenses/ for more information.
 
 
+=head1 SEE ALSO
+
+See http://support.crashplanpro.com/doku.php/api
+
+For a detailed API description.
+
 =cut
 
 1;    # End of Crashplan::Client
